pub struct MaskValues { /* private fields */ }Expand description
Represents the values of a Mask that contains some true and some false elements.
Implementations
Sourceimpl MaskValues
impl MaskValues
Sourcepub fn true_count(&self) -> usize
pub fn true_count(&self) -> usize
Returns the true count of the mask.
Sourcepub fn bit_buffer(&self) -> &BitBuffer
pub fn bit_buffer(&self) -> &BitBuffer
Returns the boolean buffer representation of the mask.
Sourcepub fn into_bit_buffer(self) -> BitBuffer
pub fn into_bit_buffer(self) -> BitBuffer
Returns the boolean buffer representation of the mask.
Sourcepub fn indices(&self) -> &[usize]
pub fn indices(&self) -> &[usize]
Constructs an indices vector from one of the other representations.
Sourcepub fn cached_indices(&self) -> Option<&[usize]>
pub fn cached_indices(&self) -> Option<&[usize]>
Returns cached index positions when this mask already has them materialized.
Unlike Self::indices, this does not build the index vector from another
representation.
Sourcepub fn slices(&self) -> &[(usize, usize)]
pub fn slices(&self) -> &[(usize, usize)]
Constructs a slices vector from one of the other representations.
Sourcepub fn cached_slices(&self) -> Option<&[(usize, usize)]>
pub fn cached_slices(&self) -> Option<&[(usize, usize)]>
Returns cached true-value ranges when this mask already has them materialized.
Unlike Self::slices, this does not build the slice vector from another
representation.
Sourcepub fn threshold_iter(&self, threshold: f64) -> MaskIter<'_>
pub fn threshold_iter(&self, threshold: f64) -> MaskIter<'_>
Return an iterator over either indices or slices of the mask based on a density threshold.